==> apiconnect-cli-util-1.0.3.tgz: package/lib/project/project.js <==
// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/

/**
 * Represents an offline APIM Project - it consistent of a collection of files
 * that defines products(as it is defined by product-schema.json) and apis (as
 * swagger format). A project can have one or more product and one or more
 * apis. Any files parsed by the project and if the format is not recognized
 * will be discarded during the load process.
 */

var path = require('path');
var async = require('async');
var fs = require('fs');
var debug = require('debug')('apiconnect-cli-util:project');
var _ = require('lodash');
var Product = require('./product');
var Api = require('./api');
var Entity = require('./entity');


/**
 * Load a file that represents a product or an api swagger file
 *
 * @constructor
 */
function Project(file) {
  this.file = path.resolve(file);
  this.projectDir = path.dirname(file);
  this.apis = [];
  this.products = [];
  this.policies = [];
}

/**
 * Initialize project - traverse the filesystem and identify the project
 * entities. This is a sync operation.
 *
 * @param cb {Function(err)} - callback function -
 *          err will be null if loading ended successfully.
 */
Project.prototype.load = function(cb) {
  var err;
  this.apis = [];
  this.products = [];
  this.policies = [];
  if (!cb) {
    debug('Assigning a default callback handler');
    cb = function(err) {
      if (err) {
        debug(err);
      }
    };
  }

  try {
    if (!fs.existsSync(this.file)) {
      err = new Error(this.file + ' does not exist.', 404);
      return cb(err);
    }
    var stat = fs.statSync(this.file);
    if (stat.isDirectory()) {
      err = new Error(this.file + ' is a directory; expected a file.', 400);//bad request
      return cb(err);
    }
    this.processEntity(this.file);
    postLoading(this);
    return cb(null);
  }
  catch (error) {
    debug(error.toString());
    return cb(error);
  }
};
/**
 * Process entity specified by <tt>file</tt>
 *
 * @param file location of the entity
 */
Project.prototype.processEntity = function(/**String*/file) {
  debug('Processing ' + file);
  var entity = new Entity(file);
  var name = entity.name();
  if (entity.entityType() === 'api') {
    this.apis.push({
      filepath$ref: file,
      name: name,
      version: entity.info().version,
      entity: new Api(entity),
    });
  }
  else if (entity.entityType() === 'product') {
    this.products.push({
      name: entity.info().name,
      version: entity.info().version,
      entity: new Product(entity),
    });
  }
};

function postLoading(project) {
  var prod;
  for (var i = 0; i < project.products.length; i++) {
    prod = project.products[i];
    var apis = prod.entity.listAllApis();
    _.each(_.keys(apis), function(apiName) {
      var api = apis[apiName];
      if (api.$ref) {
        var filename = api.$ref;
        var parentDir = filename.indexOf('/') === 0 ? '' : path.dirname(prod.entity.path) + '/';
        filename = path.resolve(parentDir, filename);
        if (!fs.existsSync(filename)) {
          console.error('File does not exist: ' + filename);
        }
        var entity = new Entity(filename);
        var name = entity.name();
        var version = entity.info().version;
        if (entity.entityType() === 'api') {
          var apiPrj = {
            name: name,
            version: version,
            entity: new Api(entity),
            filepath$ref: api.$ref,
          };
          var obj = _.find(project.apis,
            {
              name: name,
              version: version,
            });
          if (obj) {
            obj.filepath$ref = api.$ref;
            debug('Api defined by path: ' + filename +
                  ' already processed and added to project');
          }
          else {
            project.apis.push(apiPrj);
          }
        }
      }

    });
  }
}

/**
 * Project root is a file
 *
 * @return {string}
 */

Project.prototype.isRootFile = function() {
  var stat = fs.statSync(this.projectDir);
  return stat.isFile();
};

/**
 * Directory of the project.
 *
 * @returns {String}
 */
Project.prototype.directory = function() {
  if (!this.dirPath) {
    var stat = fs.statSync(this.projectDir);
    if (stat.isDirectory()) {
      this.dirPath = this.projectDir;
    }
    else {
      this.dirPath = path.dirname(this.projectDir);
    }
  }
  return this.dirPath;
};

/**
 * Return the apis entities.
 * @return {Array} array of [ {name: <name>, entity: <entity>} }  ]
 */
Project.prototype.getApis = function() {
  return this.apis;
};
/**
 * Return the api (swagger definition) named <tt>apiName</tt> or null
 * if is not found.
 *
 * @param apiName {String} name of the api
 * @param version {String|optional} version of the api
 * @return {object} the api entity
 *         {name: <name>, version: <version>, entity: <Entity>}
 */
Project.prototype.findApi = function(/**String*/ apiName, /**String*/version) {
  for (var i = 0; i < this.apis.length; i++) {
    if (this.apis[i].name === apiName) {
      if (version) {
        if (this.apis[i].version === version) {
          return this.apis[i];
        }
      }
      else {
        return this.apis[i];
      }
    }
  }
  return null;
};

Project.prototype.findApiByFilePath = function(/**String*/ filePath) {
  for (var i = 0; i < this.apis.length; i++) {
    if (this.apis[i].filepath$ref === filePath) {
      return this.apis[i];
    }
  }
  return null;
};

/**
 * Return the api (swagger definition) named <tt>apiName</tt> or null if is not
 * found. Similar to {@see #findApi},
 * but this method is looking for title that matches <tt>apiName</tt>.
 *
 * @param apiName {String} title of the api
 * @param version {String|optional} version of the api
 * @return {object} the api entity
 *         {name: <name>, version: <version>, entity: <Entity>}
 */
Project.prototype.findApiByTitle = function(apiName, version) {
  for (var i = 0; i < this.apis.length; i++) {
    if (this.apis[i].entity.content &&
      this.apis[i].entity.content.info &&
      this.apis[i].entity.content.info.title === apiName) {
      if (version) {
        if (this.apis[i].version === version) {
          return this.apis[i];
        }
      }
      else {
        return this.apis[i];
      }
    }
  }
  return null;
};
/**
 * Return project products
 *
 * @returns {Array}   [ {name: <name>, entity: <entity>} }  ]
 */
Project.prototype.getProducts = function() {
  return this.products;
};


Project.prototype.getPolicies = function() {
  return this.policies;
};

//process the project for upload
//update the product and replaces $ref with name:version for all the apis in use
//
Project.prototype.processForUpload = function() {
  var self = this;
  self.products.forEach(function(product) {
    self.processProductForUpload(product.entity);
  });
};
/**
 * Process product before uploading to apim server
 * Replaces the $ref from file:/// to name:version
 *
 * @param product {Product}
 */
Project.prototype.processProductForUpload = function(product) {
  var self = this;
  // Updating the apis in the product
  var entity = product.entity || product;
  _.keys(entity.getContent().apis).forEach(function(apiName) {
    var api = entity.getContent().apis[apiName];
    var idx = _.findIndex(self.apis, { filepath$ref: api.$ref });
    var apiInstance = self.apis[idx];
    api.name = apiInstance.name + ':' + apiInstance.version;
    delete api.$ref;
  });
};

/**
 * Find product named <tt>productName</tt>
 *
 * @param productName {string} name of the product
 * @param productVersion {string} [optional] if not specified then it will
 *        return the first product find that matches the name
 * @return {Product}
 */
Project.prototype.findProduct = function(productName, productVersion) {
  var product;
  if (productVersion) {
    product = _.find(this.products,
      {
        name: productName,
        version: productVersion,
      });
  }
  else {
    product = _.find(this.products, { name: productName });
  }
  if (product) {
    return product.entity;
  }
  return null;
};
/**
 * Validate project entries - should be invoked once the project has been
 * loaded.
 *
 * @param opts {object} - options used during validation
 *                 opts.serverValidation: {boolean}
 * @param cbFunc {Function(Array)} - callback function that receives as
 *           args an array results objects
 *           ({name: <entry_name>, status: <valid|not_valid>, err: Error});
 */
Project.prototype.validate = function(opts, cbFunc) {
  var i = 0;
  var self = this;
  var tasks = [];
  var cb = cbFunc;
  var error;
  if (typeof opts === 'function') {
    cb = opts;
    opts = {};
  }
  tasks.push(function(callback) {
    callback(null, []);
  });
  try {
    if (self.products.length === 0 && self.apis.length === 0) {
      error = new Error('Invalid product or api provided');
      return cb(error, null);
    }

    if (self.products.length === 0 && opts.productOnly) {
      error = new Error('--product-only option is not valid for an api definition');
      return cb(error, null);
    }
    for (; i < self.products.length; i++) {
      var prodValidation = (function(product) {
        return {
          invoke: function(results, callback) {
            var name = product.name;
            var path = product.entity.path;
            product.entity.validate(opts, function(err, res) {
              if (err) {
                results.push({
                  name: name,
                  path: path,
                  error: err,
                  type: 'product',
                  entry: product,
                  status: 'not_valid',
                });
              }
              else {
                results.push({
                  name: name,
                  path: path,
                  type: 'product',
                  entry: product,
                  status: 'valid',
                });
              }
              callback(null, results);
            });
          },
        };
      })(self.products[i]);   //jshint ignore:line
      tasks.push(prodValidation.invoke);
    }
    if (!opts.productOnly) {
      for (i = 0; i < self.apis.length; i++) {
        var apiValidate = (function(api) {
          return {
            invoke: function(results, callback) {
              var name = api.name;
              var path = api.entity.path;
              Api.prototype.validate.call(api.entity, opts, function(err, res) {
                if (err) {
                  results.push({
                    name: name,
                    path: path,
                    error: err,
                    type: 'api',
                    entry: api,
                    status: 'not_valid',
                  });
                }
                else {
                  results.push({
                    name: name,
                    path: path,
                    type: 'api',
                    entry: api,
                    status: 'valid',
                  });
                }
                callback(null, results);
              });
            },
          };
        })(self.apis[i]);   //jshint ignore:line
        tasks.push(apiValidate.invoke);
      }
    }

    async.waterfall(tasks, function(err, result) {
      if (cb) {
        cb(err, result);
      }
    });

  }
  catch (err) {
    debug(err);
    if (cb) {
      cb(err, null);
    }
  }
};
/**
 * Dump the project to the console
 */
Project.prototype.dump = function() {
  debug('=========APIs================');
  debug(JSON.stringify(this.apis, null, ' '));
  debug('=========Products==============');
  debug(JSON.stringify(this.products, null, ' '));
  //logger.info('=========Policies=============');
  //logger.info(JSON.stringify(this.policies, null, ' '));
};

module.exports = Project;

==> apiconnect-cli-util-3.0.3.tgz: package/lib/project/project.js <==
// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-util
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/
/*eslint brace-style: 0, max-len: 0*/

/**
 * Represents an offline APIM Project - it consistent of a collection of files
 * that defines products(as it is defined by product-schema.json) and apis (as
 * swagger format). A project can have one or more product and one or more
 * apis. Any files parsed by the project and if the format is not recognized
 * will be discarded during the load process.
 */

var path = require('path');
var async = require('async');
var f = require('util').format;
var fs = require('fs');
var debug = require('debug')('apiconnect-cli-util:project');
var _ = require('lodash');
var Product = require('./product');
var Api = require('./api');
var Entity = require('./entity');


/**
 * Load a file that represents a product or an api swagger file
 *
 * @constructor
 */
function Project(file) {
  this.file = path.resolve(file);
  this.projectDir = path.dirname(file);
  this.apis = [];
  this.products = [];
  this.policies = [];
}

/**
 * Initialize project - traverse the filesystem and identify the project
 * entities. This is a sync operation.
 *
 * @param cb {Function(err)} - callback function -
 *          err will be null if loading ended successfully.
 */
Project.prototype.load = function(cb) {
  var err;
  this.apis = [];
  this.products = [];
  this.policies = [];
  if (!cb) {
    debug('Assigning a default callback handler');
    cb = function(err) {
      if (err) {
        debug(err);
      }
    };
  }

  try {
    if (!fs.existsSync(this.file)) {
      err = new Error(f('Status: %d: The file %s does not exist.',
        404, this.file));
      return cb(err);
    }
    var stat = fs.statSync(this.file);
    if (stat.isDirectory()) {
      err = new Error(f('Status: %d: %s is a directory.  It must be a file.',
        404, this.file));
      return cb(err);
    }
    this.processEntity(this.file);
    postLoading(this);
    return cb(null);
  }
  catch (error) {
    debug(error.toString());
    return cb(error);
  }
};
/**
 * Process entity specified by <tt>file</tt>
 *
 * @param file location of the entity
 */
Project.prototype.processEntity = function(/**String*/file) {
  debug('Processing ' + file);
  var entity = new Entity(file);
  entity.info().version = stringifyVersion(entity.info().version);
  var name = entity.name();
  if (entity.entityType() === 'api') {
    this.apis.push({
      filepath$ref: file,
      name: name,
      version: entity.info().version,
      entity: new Api(entity),
    });
  }
  else if (entity.entityType() === 'product') {
    this.products.push({
      name: entity.info().name,
      version: entity.info().version,
      entity: new Product(entity),
    });
  }
};

function postLoading(project) {
  var prod;
  for (var i = 0; i < project.products.length; i++) {
    prod = project.products[i];
    var apis = prod.entity.listAllApis();
    _.each(_.keys(apis), function(apiName) {
      var api = apis[apiName];
      if (api.$ref) {
        var filename = api.$ref;
        var parentDir = filename.indexOf('/') === 0 ? '' : path.dirname(prod.entity.path) + '/';
        filename = path.resolve(parentDir, filename);
        if (!fs.existsSync(filename)) {
          console.error('File does not exist: ' + filename);
        }
        var entity = new Entity(filename);
        var name = entity.name();
        entity.info().version = stringifyVersion(entity.info().version);
        var version = entity.info().version;
        if (entity.entityType() === 'api') {
          var apiPrj = {
            name: name,
            version: version,
            entity: new Api(entity),
            filepath$ref: api.$ref,
          };
          var obj = _.find(project.apis,
            {
              name: name,
              version: version,
            });
          if (obj) {
            obj.filepath$ref = api.$ref;
            debug('Api defined by path: ' + filename +
                  ' already processed and added to project');
          }
          else {
            project.apis.push(apiPrj);
          }
        }
      }

    });
  }
}

/**
 * Project root is a file
 *
 * @return {string}
 */

Project.prototype.isRootFile = function() {
  var stat = fs.statSync(this.projectDir);
  return stat.isFile();
};

/**
 * Directory of the project.
 *
 * @returns {String}
 */
Project.prototype.directory = function() {
  if (!this.dirPath) {
    var stat = fs.statSync(this.projectDir);
    if (stat.isDirectory()) {
      this.dirPath = this.projectDir;
    }
    else {
      this.dirPath = path.dirname(this.projectDir);
    }
  }
  return this.dirPath;
};

/**
 * Return the apis entities.
 * @return {Array} array of [ {name: <name>, entity: <entity>} }  ]
 */
Project.prototype.getApis = function() {
  return this.apis;
};
/**
 * Return the api (swagger definition) named <tt>apiName</tt> or null
 * if is not found.
 *
 * @param apiName {String} name of the api
 * @param version {String|optional} version of the api
 * @return {object} the api entity
 *         {name: <name>, version: <version>, entity: <Entity>}
 */
Project.prototype.findApi = function(/**String*/ apiName, /**String*/version) {
  for (var i = 0; i < this.apis.length; i++) {
    if (this.apis[i].name === apiName) {
      if (version) {
        if (this.apis[i].version === version) {
          return this.apis[i];
        }
      }
      else {
        return this.apis[i];
      }
    }
  }
  return null;
};

Project.prototype.findApiByFilePath = function(/**String*/ filePath) {
  for (var i = 0; i < this.apis.length; i++) {
    if (this.apis[i].filepath$ref === filePath) {
      return this.apis[i];
    }
  }
  return null;
};

/**
 * Return the api (swagger definition) named <tt>apiName</tt> or null if is not
 * found. Similar to {@see #findApi},
 * but this method is looking for title that matches <tt>apiName</tt>.
 *
 * @param apiName {String} title of the api
 * @param version {String|optional} version of the api
 * @return {object} the api entity
 *         {name: <name>, version: <version>, entity: <Entity>}
 */
Project.prototype.findApiByTitle = function(apiName, version) {
  for (var i = 0; i < this.apis.length; i++) {
    if (this.apis[i].entity.content &&
      this.apis[i].entity.content.info &&
      this.apis[i].entity.content.info.title === apiName) {
      if (version) {
        if (this.apis[i].version === version) {
          return this.apis[i];
        }
      }
      else {
        return this.apis[i];
      }
    }
  }
  return null;
};
/**
 * Return project products
 *
 * @returns {Array}   [ {name: <name>, entity: <entity>} }  ]
 */
Project.prototype.getProducts = function() {
  return this.products;
};


Project.prototype.getPolicies = function() {
  return this.policies;
};

//process the project for upload
//update the product and replaces $ref with name:version for all the apis in use
//
Project.prototype.processForUpload = function() {
  var self = this;
  self.products.forEach(function(product) {
    self.processProductForUpload(product.entity);
  });
};
/**
 * Process product before uploading to apim server
 * Replaces the $ref from file:/// to name:version
 *
 * @param product {Product}
 */
Project.prototype.processProductForUpload = function(product) {
  var self = this;
  // Updating the apis in the product
  var entity = product.entity || product;
  _.keys(entity.getContent().apis).forEach(function(apiName) {
    var api = entity.getContent().apis[apiName];
    var idx = _.findIndex(self.apis, { filepath$ref: api.$ref });
    var apiInstance = self.apis[idx];
    api.name = apiInstance.name + ':' + apiInstance.version;
    delete api.$ref;
  });
};

/**
 * Find product named <tt>productName</tt>
 *
 * @param productName {string} name of the product
 * @param productVersion {string} [optional] if not specified then it will
 *        return the first product find that matches the name
 * @return {Product}
 */
Project.prototype.findProduct = function(productName, productVersion) {
  var product;
  if (productVersion) {
    product = _.find(this.products,
      {
        name: productName,
        version: productVersion,
      });
  }
  else {
    product = _.find(this.products, { name: productName });
  }
  if (product) {
    return product.entity;
  }
  return null;
};
/**
 * Validate project entries - should be invoked once the project has been
 * loaded.
 *
 * @param opts {object} - options used during validation
 *                 opts.serverValidation: {boolean}
 * @param cbFunc {Function(Array)} - callback function that receives as
 *           args an array results objects
 *           ({name: <entry_name>, status: <valid|not_valid>, err: Error});
 */
Project.prototype.validate = function(opts, cbFunc) {
  var i = 0;
  var self = this;
  var tasks = [];
  var cb = cbFunc;
  var error;
  if (typeof opts === 'function') {
    cb = opts;
    opts = {};
  }
  tasks.push(function(callback) {
    callback(null, []);
  });
  try {
    if (self.products.length === 0 && self.apis.length === 0) {
      error = new Error('The product or api is invalid.');
      return cb(error, null);
    }

    if (self.products.length === 0 && opts.productOnly) {
      error = new Error('The --product-only option is invalid for the API definition.');
      return cb(error, null);
    }
    for (; i < self.products.length; i++) {
      var prodValidation = (function(product) {
        return {
          invoke: function(results, callback) {
            var name = product.name;
            var path = product.entity.path;
            product.entity.validate(opts, function(err, res) {
              if (err) {
                results.push({
                  name: name,
                  path: path,
                  error: err,
                  type: 'product',
                  entry: product,
                  status: 'not_valid',
                });
              }
              else {
                results.push({
                  name: name,
                  path: path,
                  type: 'product',
                  entry: product,
                  status: 'valid',
                });
              }
              callback(null, results);
            });
          },
        };
      })(self.products[i]);   //jshint ignore:line
      tasks.push(prodValidation.invoke);
    }
    if (!opts.productOnly) {
      for (i = 0; i < self.apis.length; i++) {
        var apiValidate = (function(api) {
          return {
            invoke: function(results, callback) {
              var name = api.name;
              var path = api.entity.path;
              Api.prototype.validate.call(api.entity, opts, function(err, res) {
                if (err) {
                  results.push({
                    name: name,
                    path: path,
                    error: err,
                    type: 'api',
                    entry: api,
                    status: 'not_valid',
                  });
                }
                else {
                  results.push({
                    name: name,
                    path: path,
                    type: 'api',
                    entry: api,
                    status: 'valid',
                  });
                }
                callback(null, results);
              });
            },
          };
        })(self.apis[i]);   //jshint ignore:line
        tasks.push(apiValidate.invoke);
      }
    }

    async.waterfall(tasks, function(err, result) {
      if (cb) {
        cb(err, result);
      }
    });

  }
  catch (err) {
    debug(err);
    if (cb) {
      cb(err, null);
    }
  }
};
/**
 * Dump the project to the console
 */
Project.prototype.dump = function() {
  debug('=========APIs================');
  debug(JSON.stringify(this.apis, null, ' '));
  debug('=========Products==============');
  debug(JSON.stringify(this.products, null, ' '));
  //logger.info('=========Policies=============');
  //logger.info(JSON.stringify(this.policies, null, ' '));
};

function stringifyVersion(version) {
  if (typeof version === 'number') {
    return version.toString();
  }
  return version;
}

module.exports = Project;
